@@ -2,9 +2,7 @@
 # For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 # Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
 
-"""
-Various helper utilities.
-"""
+"""Various helper utilities."""
 
 from __future__ import annotations
 
@@ -65,7 +63,7 @@ def _object_type(
             yield _build_proxy_class("module", builtins)
         elif isinstance(inferred, nodes.Unknown):
             raise InferenceError
-        elif inferred is util.Uninferable:
+        elif isinstance(inferred, util.UninferableBase):
             yield inferred
         elif isinstance(inferred, (bases.Proxy, nodes.Slice)):
             yield inferred._proxied
@@ -76,7 +74,7 @@ def _object_type(
 def object_type(
     node: SuccessfulInferenceResult, context: InferenceContext | None = None
 ) -> InferenceResult | None:
-    """Obtain the type of the given node
+    """Obtain the type of the given node.
 
     This is used to implement the ``type`` builtin, which means that it's
     used for inferring type calls, as well as used in a couple of other places
@@ -102,7 +100,7 @@ def _object_type_is_subclass(
     else:
         class_seq = class_or_seq
 
-    if obj_type is util.Uninferable:
+    if isinstance(obj_type, util.UninferableBase):
         return util.Uninferable
 
     # Instances are not types
@@ -114,7 +112,7 @@ def _object_type_is_subclass(
     # issubclass(type, (object, 1)) evaluates to true
     # issubclass(object, (1, type)) raises TypeError
     for klass in class_seq:
-        if klass is util.Uninferable:
+        if isinstance(klass, util.UninferableBase):
             raise AstroidTypeError("arg 2 must be a type or tuple of types")
 
         for obj_subclass in obj_type.mro():
@@ -124,7 +122,7 @@ def _object_type_is_subclass(
 
 
 def object_isinstance(node, class_or_seq, context: InferenceContext | None = None):
-    """Check if a node 'isinstance' any node in class_or_seq
+    """Check if a node 'isinstance' any node in class_or_seq.
 
     :param node: A given node
     :param class_or_seq: Union[nodes.NodeNG, Sequence[nodes.NodeNG]]
@@ -133,13 +131,13 @@ def object_isinstance(node, class_or_seq, context: InferenceContext | None = Non
     :raises AstroidTypeError: if the given ``classes_or_seq`` are not types
     """
     obj_type = object_type(node, context)
-    if obj_type is util.Uninferable:
+    if isinstance(obj_type, util.UninferableBase):
         return util.Uninferable
     return _object_type_is_subclass(obj_type, class_or_seq, context=context)
 
 
 def object_issubclass(node, class_or_seq, context: InferenceContext | None = None):
-    """Check if a type is a subclass of any node in class_or_seq
+    """Check if a type is a subclass of any node in class_or_seq.
 
     :param node: A given node
     :param class_or_seq: Union[Nodes.NodeNG, Sequence[nodes.NodeNG]]
@@ -243,7 +241,7 @@ def class_instance_as_index(node: SuccessfulInferenceResult) -> nodes.Const | No
 
 
 def object_len(node, context: InferenceContext | None = None):
-    """Infer length of given node object
+    """Infer length of given node object.
 
     :param Union[nodes.ClassDef, nodes.Instance] node:
     :param node: Node to infer length of
@@ -277,7 +275,7 @@ def object_len(node, context: InferenceContext | None = None):
         )
         raise InferenceError(message)
 
-    if inferred_node is None or inferred_node is util.Uninferable:
+    if inferred_node is None or isinstance(inferred_node, util.UninferableBase):
         raise InferenceError(node=node)
     if isinstance(inferred_node, nodes.Const) and isinstance(
         inferred_node.value, (bytes, str)
@@ -302,7 +300,7 @@ def object_len(node, context: InferenceContext | None = None):
         ) from e
 
     inferred = len_call.infer_call_result(node, context)
-    if inferred is util.Uninferable:
+    if isinstance(inferred, util.UninferableBase):
         raise InferenceError(node=node, context=context)
     result_of_len = next(inferred, None)
     if (
