@@ -2,27 +2,23 @@
 # For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 # Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
 
-"""Transform utilities (filters and decorator)"""
+"""Transform utilities (filters and decorator)."""
+
+from __future__ import annotations
 
 import typing
+from collections.abc import Iterator
 
 import wrapt
 
-from astroid import bases, util
 from astroid.exceptions import InferenceOverwriteError, UseInferenceDefault
 from astroid.nodes import NodeNG
-from astroid.typing import InferFn
+from astroid.typing import InferenceResult, InferFn
 
-InferOptions = typing.Union[
-    NodeNG, bases.Instance, bases.UnboundMethod, typing.Type[util.Uninferable]
-]
-
-_cache: typing.Dict[
-    typing.Tuple[InferFn, NodeNG], typing.Optional[typing.List[InferOptions]]
-] = {}
+_cache: dict[tuple[InferFn, NodeNG], list[InferenceResult] | None] = {}
 
 
-def clear_inference_tip_cache():
+def clear_inference_tip_cache() -> None:
     """Clear the inference tips cache."""
     _cache.clear()
 
@@ -30,8 +26,8 @@ def clear_inference_tip_cache():
 @wrapt.decorator
 def _inference_tip_cached(
     func: InferFn, instance: None, args: typing.Any, kwargs: typing.Any
-) -> typing.Iterator[InferOptions]:
-    """Cache decorator used for inference tips"""
+) -> Iterator[InferenceResult]:
+    """Cache decorator used for inference tips."""
     node = args[0]
     try:
         result = _cache[func, node]
